#
# Copyright (c) 1992 The Ohio State University.
# All rights reserved.
#
# Redistribution and use in source and binary forms are permitted
# provided that: (1) source distributions retain this entire copyright
# notice and comment, and (2) distributions including binaries display
# the following acknowledgement:  ``This product includes software
# developed by The Ohio State University and its contributors''
# in the documentation or other materials provided with the distribution
# and in all advertising materials mentioning features or use of this
# software. Neither the name of the University nor the names of its
# contributors may be used to endorse or promote products derived
# from this software without specific prior written permission.
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#

$LOOK_FOR = "5348";				# a run entry that should exist
$copy_dir = "/n/spice/0/bu";			# where to put copies
$warn_hosts = "brachiosaur skiing";		# hosts to warn with rwall
$warn_people = "romig sullivan cis.staff";	# people to warn with mail

$options = "d";
$usage = "usage: backup-db-fixer [-d]\n";

require 'global.defs';
require 'local.defs';
require $yagrip_perl;
require $backuplib_perl;

&read_config();			# Read backup.config file

#
# copy database to bu.tar
#
print "chdir $db_dir\n" if defined($opt_d);
chdir $db_dir || die "cd $db_dir failed ($!)\n";

print "gtar ... new.tar\n" if defined($opt_d);
if (system("/usr/local/bin/gtar -c -S -v -f $copy_dir/new.tar .")) {
    die "gtar -c of backup-db failed ($!)\n";
}

if (&database_is_ok()) {
    print "chdir $copy_dir\n" if defined($opt_d);
    chdir $copy_dir || die "cd $copy_dir failed ($!)\n";

    #
    # rename saved copy to be current good copy
    #
    if (-f "save.tar") {
	print "rename save.tar old.tar\n" if defined($opt_d);
	rename("save.tar", "old.tar") || 
	  die "rename save.tar old.tar failed ($!)\n";
    }

    print "rename new.tar save.tar\n" if defined($opt_d);
    rename("new.tar", "save.tar") || die "rename new.tar save.tar failed ($!)\n";

    #
    # Update the timestamp
    #
    open(TIMESTAMP, "> save.time") || die "can't open save.time ($!)\n";
    print TIMESTAMP `date`;
    close(TIMESTAMP);
} else {
    print "cd $copy_dir\n" if defined($opt_d);
    chdir $copy_dir || die "cd $copy_dir failed ($!)\n";

    #
    # clean up
    #
    print "rename new.tar time.tar\n" if defined($opt_d);
    rename("new.tar", time . ".tar");

    print "cd $db_dir/..\n" if defined($opt_d);
    chdir("$db_dir/..") || &fooey("cd $db_dir/.. failed ($!)\n");

    print "rename backup-db DeleteMe\n" if defined($opt_d);
    rename("backup-db", "DeleteMe") || &fooey("rename backup-db DeleteMe failed ($!)\n");

    print "rm -rf DeleteMe\n" if defined($opt_d);
    system("rm -rf DeleteMe");

    print "mkdir backup-foo\n" if defined($opt_d);
    mkdir("backup-foo", 0664) || &fooey("mkdir backup-foo failed ($!)\n");

    print "cd backup-foo\n" if defined($opt_d);
    chdir("backup-foo") || &fooey("cd backup-foo failed ($!)\n");

    print "gtar save.tar\n" if defined($opt_d);
    if (system("/usr/local/bin/gtar -x -p -v -f $copy_dir/save.tar")) {
	&fooey("gtar extract failed ($!)\n");
    }

    print "cd ..\n" if defined($opt_d);
    chdir("..") || &fooey("cd .. failed ($!)\n");

    print "rename backup-foo backup-db\n" if defined($opt_d);
    rename("backup-foo", "backup-db") || &fooey("rename backup-foo backup-db failed ($!)\n");

    #
    # Warn people
    #
    $timestamp = `cat $copy_dir/save.time`;
    chop($timestamp);

    $message = "The backup database has been trashed (again).
It has been fixed automatically from a copy made at
$timestamp, but someone needs to check the status
of backups done lately to see if they need to be redone.

Call Steve at home if necessary!.\n";

    &warn_folks($message);
}

exit(0);

sub database_is_ok {
    local($result);

    &open_db(*RUN, $run_db_file, $db_mode) ||
      die "Can't open the run database file '$run_db_file' in database-check\n";

    $result = defined($RUN{$LOOK_FOR});

    &close_db(*RUN);

    return($result);
}

sub warn_folks {
    local($message) = @_;

    open(RWALL, "|/usr/etc/rwall $warn_hosts");
    print RWALL $message;
    close(RWALL);

    open(MAIL, "|/usr/lib/sendmail -t");
    print MAIL "To: $warn_people
Subject: Backup database trashed again!

";
    print MAIL $message;
    close(MAIL);
}

sub fooey {
    local($error) = @_;
    local($message);

    $message = "The backup database was trashed again, and I
couldn't automatically fix things because of an error:

    $error
Please restore the backup database manually ASAP.  If necessary, 
call Steve at home (263-7663).  Thanks.
";
    
    &warn_folks($message);
    die $error;
}
