#
# Copyright (c) 1990,1991,1992 The Ohio State University.
# All rights reserved.
#
# Redistribution and use in source and binary forms are permitted
# provided that: (1) source distributions retain this entire copyright
# notice and comment, and (2) distributions including binaries display
# the following acknowledgement:  ``This product includes software
# developed by The Ohio State University and its contributors''
# in the documentation or other materials provided with the distribution
# and in all advertising materials mentioning features or use of this
# software. Neither the name of the University nor the names of its
# contributors may be used to endorse or promote products derived
# from this software without specific prior written permission.
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#

#
# usage: all_backups
#
# Abstract:	This is the top level backup program.  'backup' backs
#		up one directory/file system and updates the tardates
#		file.  'do_backups' backs up all the backupable file
#		systems on a particular host according to the fstab
#		file.  'all_backups' uses 'do_backups' to do the
#		backups on all of the hosts for a particular (chain,
#		level).
#
# (fixme) Eventually we ought to do things like check to see whether
#	  the drive is in use when we choose a drive, etc.
#

# Alphabet soup...sigh.
$options = "Ac:C:d:ef:g:hHi:l:m:Mpr:R:st:T:Vx:";
$usage = 
"usage: all-backups [-c chain] [-l level] [-g hosts-and-groups] [-f tapedrive]
    [-h] [-p] [-T time] [-R yes/no] [-C check-time] [-m disk|tape|both] 
    [-i tag] [-x tag] [-A] [-V] [-r run-id] [-t tape-id] [-d debuglevel]
    [-M] [-e] [-s]

    -c chain		sets chain number of this backup
    -l level		sets level number (or name)
    -g hosts-and-groups	set which hosts (or group of hosts) to do backups on
    -f tapedrive	host:device to use
    -h			display this message
    -p			do backups in parallel
    -T time		when to start the backup
    -R yes/no		whether to run the backups in single user mode
    -C checktime        when to check the tapes
    -m tape|disk|both	where should archives finally reside?
    -i tag		include only file systems with this tag
    -x tag		avoid file systems with this tag
    -A			auto-pilot mode - don't be user friendly
    -V			assume that the tape is mounted already
    -r run-id		use this run ID number
    -t tape-id          use this tape rather than picking one
    -d debuglevel	turn on debugging output
    -M			don't verify that the tape is loaded before 
			scheduling with at
    -H			halt the system after completing the backup
    -e			Don't eject tape when done
    -s			scavenge mode - use any wildcard promised tape
";

require 'global.defs';
require 'local.defs';

require $yagrip_perl;
require $backuplib_perl;
require $lib_db_extra_perl;
require $lib_deadman_perl if $use_deadman_stuff;

&read_config();			# Read backup.config file

&init();			# Deal with args and get necessary info...

#
# Rewrite the tape label, leaving it positioned after it, ready 
# for backups.  We do this both when the run is scheduled and when 
# it starts: at setup to check that the tape is writeable, and again
# later to ensure that it is positioned correctly.
#
&relabel_tape() if $needs_tape && ! $send_mail_only && (! $scavange || $when eq "now");

if ($when ne "now") {
    if ($checktime ne "") {	# If we're supposed to check tapes, 
        #
        # Rewind the tape (if there is one) so that the darned thing 
	# will be at BOT for check-tape.  This is safe since we're 
        # going to restart all-backups from an at job anyway, so the
        # label will be rechecked and rewritten and everything will 
        # be fine...
        #
	if ($needs_tape && ! $send_mail_only) {
	    if (&tape_rewind($rsh, $tape_dev)) {
		print stderr "warning (all-backups): can't rewind $tape_host:$tape_dev\n";
	    }
	}

	&schedule_tape_check();	# then schedule it.
    }

    #
    # unlock the tape device
    #
    if (defined($tape_device_lock_name)) {
        &unlock($tape_device_lock_name);
    }

    &create_run_db("scheduled", $when);# Add an entry for this run
    # don't worry about the status - it dies if it fails!

    if ($opt_R eq "yes") {
        &reschedule_backup(1);	# Deal with reboots somehow
        # NOTREACHED
    } else {
        &reschedule_backup(0);	# Do delayed start or reboot if needed
        # NOTREACHED
    }
}

#
# invoke the deadman watcher, no timeout.
#
&deadman(0, $run_id) if $use_deadman_stuff;

&make_log_entry("all-backups, starting run $run_id (chain $chain,$level hosts $host_string drive $drive_name tape $tape_id $exclude_string $include_string $scavange_string $media_string $parallel_string $eject_string)");

&update_tape_db();		# Update the entry for this tape
&update_run_db("started");	# Add an entry for this run

foreach $host (@hostlist) {	# Foreach host in the @hostlist...
    print "backing up $host\n";

    if ($host eq $hostname) {
	$bu_rsh = "";
    } else {
	$bu_rsh = "$rsh_prog $host -n";
    }

    $cmd = "$bu_rsh $do_backup_prog -c $chain -l $level $tape_string $debug_string $include_string $exclude_string $media_string -r $run_id $parallel_string";

    print "all_backups: '$cmd'\n" if $opt_d & $DEBUG_SYSTEM;
 
    if (defined($opt_p)) {
	if (($pid = fork) == 0) { # If in parallel, fork so they all
				  # run in parallel
	    exec($cmd) &&
	      print stderr "warning (all-backups): command '$cmd' failed!\n";
	}

	push(@pid_list, $pid);	# Better keep track of the pids, though...
    } else {
	system($cmd) &&		# If not in parallel, run one at a time
	  print stderr "warning (all-backups): command '$cmd' failed!\n";
    }
}

#
# Orf.  If we are doing things in parallel, then we need to watch for
# backups and copy them to tape as they finish.
#
&parallel_processing() if defined($opt_p);

&finish_run_db();		# Update the run db entry since we're done.

				# Rewind and eject the tape...
if (defined($opt_e)) {
	&tape_rewind($rsh, $tape_dev) if $needs_tape;
} else {
	&tape_offload($rsh, $tape_dev) if $needs_tape;
}

#
# unlock the tape device
#
if (defined($tape_device_lock_name)) {
    &unlock($tape_device_lock_name);
}

exit 0;				# Done!

# NOT REACHED

#
# Get the list of host/buids for this run.  Finds list of backups for all hosts 
# in this run whose disk backups are done and whose tape backups haven't been 
# started.
#
sub get_buids_for_run {
    local($run_id) = @_;
    local($host, @ready_to_copy, $in_progress, %DB, $bu_value, %bu_array);
    local(%replace, %add, %result);

    @ready_to_copy = ();
    $in_progress = 0;

    foreach $host (@hostlist) {
	#
	# Get list of BUID's for this hosts part in the run...
	#
	if (! &update_db($run_host_file . $host,
			 $run_host_file . $host,
			 $run_id, "run_db", $DB_READ,
			 *replace, *add, *result)) {
            @buid_list = split(/,/, $result{"buid_list"});
	} else {
	    @buid_list = ();
	}

	#
	# Find the ones that are ready to be copied...
	#
	$DB_NAME = $backup_host_file . $host;
	$DB_TYPE = "backup_host";
	&open_db(*DB, $DB_NAME, $db_mode) ||
	  &sigh(sprintf("error (all-backups): can't open the database %s on %s",
		      $backup_host_file . $host,
		      $hostname));

	foreach $buid (@buid_list) {
	    &read_db(*DB, $buid, *bu_value, *bu_array);

	    # if disk backup has started, then incr in_progress
	    if ($bu_array{"d-flags"} eq "started") {
		$in_progress++;
	    }

	    # if disk backup is done and tape backup wasn't started, then
	    # add this to the copy list
	    if ($bu_array{"d-flags"} eq "done" && $bu_array{"flags"} eq "") {
		push(@ready_to_copy, join($;, $host, $buid, $bu_array{"afile"}));
		$in_progress++;
	    }
	}

	&close_db(*DB, $DB_NAME);
    }

    return($in_progress, @ready_to_copy);
}

sub update_budb_for_host {
    local($host, $buid, $tape_id, $filenumber, $flags, $delete) = @_;
    local($budb_name, %BUDB);

    $budb_name = "$backup_host_file$host";

    if (&lock($budb_name) != $LOCK_OK) {
	&sigh("all-backups: can't lock '$budb_name' in update_budb_for_host\n");
    }

    &open_db(*BUDB, $budb_name, $db_mode) ||
      &sigh("all-backups: can't open '$budb_name' in update_budb_for_host\n");

    if (! &read_db(*BUDB, $buid, *budb_value, *budb_array) || $budb_value eq "") {
	&sigh("all-backups: '$budb_name' entry '$buid' has no value.\n");
    }

    $budb_array{"tflist"} = "$tape_id.$filenumber";
    $budb_array{"flags"} = $flags;
    $budb_array{"accuracy"} = $tape_host_accuracy;
    if ($delete) {
	delete $budb_array{"afile"};
	delete $budb_array{"d-flags"};
    }

    &write_db(*BUDB, $buid, *budb_array, $budb_name, "backup_host") ||
      &sigh("all-backups: can't write entry '$buid' in '$budb_name'\n");

    &close_db(*BUDB, $budb_name);

    &unlock($budb_name);
}


sub init {
    local(%TAPEDB, $tdb_value, %tdb_array);

    #
    # Make sure that we clean up after ourselves if someone wants to kill us.
    #
    $SIG{'INT'} = 'handle_interrupts';
    $SIG{'HUP'} = 'handle_interrupts';
    $SIG{'TERM'} = 'handle_interrupts';
    $SIG{'QUIT'} = 'handle_interrupts';

    # 
    # Deal with args...we define *_string things to use as args for progs 
    # called from here so that we can pass on the right values (eg, -d
    # or "", etc). 
    #
    die $usage if ! &getopt($options);
    die $usage if $#ARGV != -1;
    die $usage if defined($opt_h);

    if (defined($opt_p)) {
	$parallel_string = "-p";
    } else {
	$parallel_string = "";
    }

    if (! defined($opt_m)) {
	$opt_m = "tape";
    }

    if ($opt_m ne "tape" && $opt_m ne "disk" && $opt_m ne "both") {
	&sigh("error (all-backups): media type $opt_m isn't one of tape, disk or both.\n$usage");
    }
    $media_string = "-m $opt_m";

    if ($opt_m eq "tape" || $opt_m eq "both") {
	$needs_tape = 1;
    } else {
	$needs_tape = 0;
    }

    if (defined($opt_s)) {
	$scavange = 1;
	$scavange_string = "-s";
	if (! $needs_tape) {
	    &sigh("error (all-backups): scavange mode selected but we aren't backing up to tape in this run.\n");
	}

	if (defined($opt_t)) {
	    &sigh("error (all-backups): scavange mode selected but you also specified a tape to use?\n");
	}
    } else {
	$scavange = 0;
	$scavange_string = "";
    }

    if (defined($opt_d)) {
	$debug_string = "-d $opt_d";
    } else {
	$debug_string = "";
    }

    if (defined($opt_x)) {
	$exclude_string = "-x $opt_x";
    } else {
	$exclude_string = "";
    }

    if (defined($opt_i)) {
	$include_string = "-i $opt_i";
    } else {
	$include_string = "";
    }

    if (defined($opt_e)) {
	$eject_string = "-e";
    } else {
	$eject_string = "";
    }

    $halt_after_backup = defined($opt_H);
    $send_mail_only = defined($opt_M);
    $autopilot = defined($opt_A);

    #
    # Get the chain, level to work out of.
    #
    $chain = &get_chain();
    $level = &get_level();

    #
    # Which systems should we do?  Get list of groups...
    #
    if (defined($opt_g)) {
	@group_list = split(/,/, $opt_g);
    } else {
	@group_list = split(/[ \t,]+/, 
			    &askdef("Which hosts do you want to do", "all", 
"List the names of the hosts that are to be backed up in this run.
You can also give host group names, as defined in your backup.config file.\n"));
    }

    #
    # Resolve the group list into a host list, using aliases where needed...
    # (fixme) how to deal with misspelled aliases?  don't really want to wait 
    # till we try to do a backup, do we?
    #
    @hostlist = ();
    foreach $group (@group_list) {
	if (defined($group_aliases{$group})) {
	    foreach $host (split(/[ \t]+/, $group_aliases{$group})) {
		push(@hostlist, $host);
	    }
	} else {
	    push(@hostlist, $group);
	}
    }

    #
    # Construct the text copy of the list of hosts...
    #
    $host_string = join(',', @hostlist);

    if (defined($opt_T)) {
	$when = $opt_T;
    } else {
	$when = &askdef("When do you want this backup to start", "now", 
"Give the time that you want the backup to start at.  Use the word 'now'
if you want the backup to run right away.  If you want to schedule it for
a later time, enter a time like '930' for 9:30 A.M., or 1345 for 1:45 P.M.
The time can be any form that your system's 'at' command can take - it
usually has to be 24 hour clock, with no punctuation characters.\n" );
    }

    if (defined($opt_C)) {
	$checktime = $opt_C;
    } elsif ($when ne "now" && $needs_tape) {
	$checktime = &askdef(
"When do you want to check that the tapes are loaded correctly?
(Just type return if you do not want to check)", "",
"Give the time that you want the system to check to see that the tape is 
loaded in the drive.  Enter nothing if you do not want it to check.  
The time can be any form that your system's 'at' command can take - it
usually has to be 24 hour clock, with no punctuation characters (930 for 
9:30 A.M., 1345 for 1:45 P.M.)\n" );
    }

    if ($when eq "now" && $checktime ne "") {
	print stderr "warning: it is silly to do the backup right away but also tell me to check the tapes
later.  I'm ignoring the extra tape checking.\n";
	$checktime = "";
    }

    if (! defined($opt_R)) {
	$opt_R = &askdef("Do you want to run this backup in single user mode", "no",
"Type 'yes' if you want the system to reboot and run the backup in single
user mode.  This should guarantee that your backups are 'good', since the 
file systems will be quiescent.  If you want to run the backups in multi-user
mode, just type 'no'.\n");
    }

    if ($opt_R eq "yes" && $host_string ne $hostname) {
	&sigh( 
"error (all-backups): you can only schedule single-user backups on 
the (single) host you want to backup.  Login there and try again.\n");
    }

    #
    # Get a run ID number so we can tell the folks what to label this tape...
    #
    if (defined($opt_r)) {
	$run_id = $opt_r;
    } else {
	$run_id = &get_id($run_db_file);
    }

    #
    # Ask for the name of the tape host and device to use, get a tape
    # number to use, ask for it to be mounted, and verify that it is
    # mounted correctly.  Repeat until the user either gets it right or
    # gives up in disgust.
    # 
    if ($needs_tape) {
        if (defined($opt_t)) {
	    $tape_id = $opt_t;

	    # fixme - create a 'promise' routine, do the confirmation 
	    # separately, check to see that the tape and drive match here 
	    # rather than later, so that we don't have to look up tape_type 
	    # and format later when we know that the tape and drive match.

	    &sigh("can't lock $tape_db_file in -t handling, timed out\n")
	      if &lock($tape_db_file) != $LOCK_OK;

	    &open_db(*TAPEDB, $tape_db_file, $db_mode) ||
		&sigh("all-backups: cannot open $tape_db_file\n");

	    &read_db(*TAPEDB, $tape_id, *tdb_value, *tdb_array) ||
		&sigh("There's no tape with id '$tape_id'.\n");

	    #
            # If we're not in autopilot mode, and the tape isn't free, then 
	    # ask for confirmation before using the tape.  If we are in auto
            # pilot mode, we assume that everything's ok (since the all-backups 
	    # that scheduled us must've thought that this tape was ok to use).
	    #
	    if (! $autopilot && 
		$tdb_array{"flags"} ne "free" &&
		! &confirm_tape_usage($tape_id, $tdb_array{"run_id"})) {
		&sigh("");
	    }	

	    #
	    # Need to promise this tape so that it isn't reused...
	    #
	    $tdb_array{"flags"} = "promised";
	    $tdb_array{"used_date"} = time;
	    $tdb_array{"run_id"} = $run_id;

	    &write_db(*TAPEDB, $tape_id, *tdb_array, $tape_db_file, "tape_db") ||
		&sigh("all-backups: couldn't write $tape_db_file\n");

	    &close_db(*TAPEDB, $tape_db_file);

	    &unlock($tape_db_file);

	    $this_tape_type = $tdb_array{"type"};
	    $this_tape_format = $tdb_array{"format"};

	    ($drive_name, $tape_user, $tape_host, $tape_dev) = 
	       &get_tape_name($this_tape_type, $this_tape_format, "read");
        } else {
	    ($drive_name, $tape_user, $tape_host, $tape_dev) = 
	       &get_tape_name("", "", "");
        }

        # various manipulations on the drive name
        # create $rsh for doing remote shell things

	$rsh = &make_rsh_cmd($tape_user, $tape_host);

        #
        # make a fake entry for this drive, just to make life easier 
	# when we need to look things up.
        #
	# fixme - hell, just require that they define all drives 
	# in the config file.
	#
        if (! defined($tape_device{$drive_name})) {
    	    ($tape_type{$drive_name}, $tape_reads{$drive_name},
	     $tape_writes{$drive_name}, $tape_status_ok{$drive_name}) =
	      &get_tape_extra_info($drive_name);
        }

	if ($tape_status_ok{$drive_name}) {
	    $tape_host_accuracy = "high";
	} else {
	    $tape_host_accuracy = "so-so";
        }

	#
	# If we are using the drive now then lock the drive.
	#
	if ($needs_tape && ! $send_mail_only && 
	    (! $scavange || $when eq "now")) {

	    $tape_device_lock_name = "$lock_dir/tape_device_$drive_name";
	    &sigh("error (all-backups): cannot lock tape $drive_name\n")
	      if &lock($tape_device_lock_name, $tape_device_timeout) != $LOCK_OK;
	}

        # pant, pant - got the drive name.  if don't know tape, choose a 
	# tape to match the drive...

	if (! defined($tape_id)) {

	    if ($scavange) {
		if ($when eq "now") {
		#
		# scavange mode - check to see what tape is in the drive.
		# if it is promised to *, use it.  else eject it and try 
		# again till we get one we can use, or no tape is in the 
		# drive
		#
		if (&scavange_for_tape($rsh, $tape_dev, *tape_id)) {
		    &sigh("error (all-backups): scavange mode could not find a usable tape\n");
		}

		print "This run will use tape $tape_id\n";
	        }
	    } else {
		#
		# if we don't know what tape we are using, choose one to
		# match the drive type and read/write formats
		#
		$tape_id = &choose_tape($tape_type{$drive_name}, 
					 $tape_reads{$drive_name}, 
					 $tape_writes{$drive_name}, 
					 $tape_location{$drive_name},
					 1,
					 $run_id);

		if ($tape_id == -1) {
		    &sigh("error (all-backups): there are no free tapes available.  Try to free some with run-clean-db, or add some to the tape database with add-tape.\n");
		}
	    }

	    #
	    # Need to know the usage count, type and format for this 
	    # tape.  No, I don't know why.  Seems silly, since we know 
	    # that this tape was chosen one way or the other for this 
	    # drive.
	    # fixme
	    #
	    if (! $scavange || $when eq "now") {
	    if (&update_db($tape_db_file, $tape_db_file, $tape_id,
			   "tape_db", $DB_READ, *replace, *add,
			   *result)) {
		&sigh("error (all-backups): tape DB read error, no entry for $tape_id\n");
	    }
	    $this_tape_type = $result{"type"};
	    $this_tape_format = $result{"format"};
	    }
	}

#        if (! &tape_and_drive_match($this_tape_type, $this_tape_format, "read", $drive_name)) {
#	    &sigh("error (all-backups): the tape $tape_id and drive $drive_name are not compatible.
#Please choose a matching set.\n");
#        }

	if ($tape_user ne "") {
	    $tape_string = "-f $tape_user@$tape_host:$tape_dev -t $tape_id";
	} else {
	    $tape_string = "-f $tape_host:$tape_dev -t $tape_id";
	}

        if (! $scavange || $when eq "now") {
	    $tape_id_string = "-t $tape_id";
	} else {
	    $tape_id_string = "";
	}
    }

    print "This is backup run # $run_id\n";

    #
    # Verify that the tape is loaded correctly if we need a tape and 
    # we're in auto pilot mode or not in mail mode
    #
    if ($needs_tape && ! $scavange) {
        if (! $autopilot && $send_mail_only) {
	    print 
"This backup run will use tape $tape_id.  Please mount it in drive $drive_name
before the backup starts.\n";
	} else {
verify_tape_loop:
	    {
		if (! &verify_tape($tape_id, $tape_host, $tape_dev, 
				   $rsh, $this_tape_type)) {
		    if ($autopilot) {
			&sigh(
"error (all-backups): I'm on autopilot and either the wrong tape has been mounted or I can't read the tape label! Bailing out!\n");
	            } else {
		        redo verify_tape_loop;
		    }
		}
	    }
	}
    }

    if ($config_name ne "") {
	$id_label = "[$config_name]";
    } else {
	$id_label = "";
    }

    if ($when eq "now" && $opt_R eq "yes") {
	&sigh("error (all-backups): you must use a time other than 'now' if you want to run the backup in single-user mode.\n");
    }

    if ($when eq "now" && $send_mail_only) {
	&sigh("error (all-backups): you must reschedule the backup for some time in the future if you don't want to verify the tape now.\n");
    }
}


#
# Use AT to schedule a check to see whether the tapes are mounted or not, 
# and if not, to bitch and moan about it to someone.
#

sub schedule_tape_check {
    local($cmd);

    chdir("/tmp") || &sigh("error (all-backups): chdir /tmp failed");

    $cmd = "| $at_prog $checktime";

    print "all-backups: open '$cmd'\n" if $opt_d & $DEBUG_SYSTEM;

    warn "warning (all-backups): can't lock $at_lock in schedule_tape_check, timed out\n" 
      if &lock($at_lock) != $LOCK_OK;
    open(AT, $cmd) || 
      &sigh("error (all-backups): can't start 'at' to schedule tape checking");

    print AT "$tape_check_prog -f $drive_name -t $tape_id > /dev/null 2>&1\n" ||
      &sigh("error (all-backups): Can't create 'at' job to check the tape.");

    close(AT);
    &sigh("error (all-backups): couldn't create 'at' job to check the tapes ($?)") if $? != 0;
    &unlock($at_lock);

    print "Scheduled tape checker at $checktime\n";
}


#
# Use 'at' to reschedule a backup, possibly for after we reboot.
#
sub reschedule_backup {
    local($single_user) = @_;
    local($cmd, $extension);

    chdir("/tmp") || &sigh("error (all-backups): chdir /tmp failed");

    $cmd = "| $at_prog $when";

    print "all-backups: open '$cmd'\n" if $opt_d & $DEBUG_SYSTEM;

    warn "warning (all-backups): can't lock $at_lock in schedule_backup, timed out\n" 
      if &lock($at_lock) != $LOCK_OK;

    delete $ENV{'TZ'};

    open(AT, $cmd) || 
      &sigh("error (all-backups): can't start 'at' to reschedule things");

    #
    # In single user mode backup, the boot rc files look for a 
    # file named /backup, so the extension has to be "".  This poses a 
    # problem on HP clusters where / is shared - /backup is not a unique 
    # name, so you should turn it into a CDF to make things work right.
    #
    if ($single_user) {
        $extension = "";
    } else {
        $extension = ".$run_id";
    }

    print AT "$cat_prog >/backup$extension <<EOF\n" ||
     &sigh("error (all-backups): can't create 'at' job");
    print AT "#! /bin/csh -f -b\n\n" ||
     &sigh("error (all-backups): can't create 'at' job");

    if ($single_user) {
        print AT "$backup_init_prog\n" ||
          &sigh("error (all-backups): can't create 'at' job");
    }

    if (! -d $output_dir) {
	mkdir($output_dir, $default_dir_mode) || 
	  &sigh("error (all-backups): $output_dir doesn't exist and I couldn't create it ($!)\n");
    }

    $command = "$all_backup_prog -A -c $chain -l $level -g $host_string -f $drive_name $tape_id_string -T now -R no -V $debug_string $exclude_string $include_string $scavange_string $media_string -r $run_id $parallel_string $eject_string ";

    print AT 
"$cat_prog >/tmp/backup-output$extension <<FOE
Subject: $id_label run $run_id tape $tape_id chain $chain,$level backups on $host_string
To: $mail_to

$command

FOE\n" ||
      &sigh("error (all-backups): can't create 'at' job");

    if ($single_user) {
	$tee = "|& $tee_prog -a $output_dir/$run_id";
    } else {
	$tee = ">>& $output_dir/$run_id";
    }

    print AT 
"$command $tee\n" ||
     &sigh("error (all-backups): can't create 'at' job");

    print AT "$egrep_prog '^usage|warn|error|info' $output_dir/$run_id >> /tmp/backup-output$extension\n\n";
    print AT "cat $output_dir/$run_id >> /tmp/backup-output$extension\n";

    if ($single_user) {
        print AT "$backup_cleanup_prog\n" || 
            &sigh("error (all-backups): can't create 'at' job");
    } else {
	print AT "$sendmail_prog -t < /tmp/backup-output$extension\n" ||
          &sigh("error (all-backups): can't create 'at' job");
	print AT "$rm_prog -f /tmp/backup-output$extension\n" ||
          &sigh("error (all-backups): can't create 'at' job");
    }

    if ($halt_after_backup) {
	print AT "$halt_program $halt_arguments\n" ||
	  &sigh("error (all-backups): can't create 'at' job");
    }
  
    #
    # End the command file, and make it executable.
    #
    print AT "EOF\n" ||
      &sigh("error (all-backups): can't create 'at' job");
    print AT "$chmod_prog 500 /backup$extension\n" ||
      &sigh("error (all-backups): can't create 'at' job");

    if ($single_user) {
        print AT "$fastboot_prog\n" ||
          &sigh("error (all-backups): can't create 'at' job");
    } else {
  	print AT "/backup$extension\n" ||
          &sigh("error (all-backups): can't create 'at' job");
        print AT "$rm_prog -f /backup$extension\n" ||
          &sigh("error (all-backups): can't create 'at' job");
    }

    close(AT);
    &sigh("error (all-backups): couldn't create 'at' job to run the backup ($?)") if $? != 0;
  
    &unlock($at_lock);

    print "Rescheduled $chain,$level of $host_string at $when\n";
    
    if ($single_user && $warn_prog ne "") {
	$cmd = "$warn_prog $chain $level $when";
	print "all-backups: system '$cmd'\n" if $opt_d & $DEBUG_SYSTEM;
	system($cmd);
    }

    &make_log_entry("all-backups, scheduled run $run_id for $when (chain $chain,$level hosts $host_string drive $drive_name tape $tape_id  $exclude_string $include_string $scavange_string $media_string $parallel_string $eject_string)");

    exit(0);
}


#
# Rewind and reposition the tape after the dd is done...argh, since 
# we can't always write a tape after rew, fsf 1, we'll just rewrite 
# the darned label from the stuff read above...
#
sub relabel_tape {
    local($label);
    local(%replace, %add, %result);

    $label = "$tape_magic T# $tape_id\nchain $chain level $level drive $tape_host:$tape_dev run $run_id\n$host_string\n";

    if (&tape_put_label($rsh, $tape_dev, $label)) {
        &sigh("error (all-backups): can't rewrite tape label.  Argh...\n");
    }

    if (&update_db($tape_db_file, $tape_db_file, $tape_id, "tape_db",
		   $DB_READ, *replace, *add, *result)) {
	&sigh("error (all-backups): tape DB read error, no entry for tape $tape_id\n");
    }

    %replace = ();
    %replace = ("format",	$tape_writes{$drive_name});
    %add = ();

    if (&update_db($tape_db_file, $tape_db_file, $tape_id, "tape_db",
		   $DB_UPDATE, *replace, *add, *result)) {
	&sigh("error (all-backups): tape DB update error, no entry for tape $tape_id\n");
    }
}


#
# Update the tape database - uses, drive, date and flag
#

sub update_tape_db {
    local(%replace, %add, %result);

    return if ! $needs_tape;

    if (&update_db($tape_db_file, $tape_db_file, $tape_id, "tape_db",
		   $DB_READ, *replace, *add, *result)) {
	&sigh("error (all-backups): tape DB read error, no entry for tape $tape_id\n");
    }

    %replace = ();
    %replace = ("uses",		$result{"uses"} + 1,
		"used_date",	time,
		"drive",	"$tape_host:$tape_dev",
		"flags",	"used",
		"run_id",	$run_id,
  		"file_number",	1,
		"accuracy",	$tape_host_accuracy);
    %add = ();

    if (&update_db($tape_db_file, $tape_db_file, $tape_id, "tape_db",
		   $DB_UPDATE, *replace, *add, *result)) {
	&sigh("error (all-backups): tape DB update error, no entry for tape $tape_id\n");
    }
}


sub parallel_processing {
    local(@ready_to_copy);	# list of backups that are ready to be copied to tape
    local($in_progress);	# number of backups that are in progress now
    local(%hitlist);		# list of files to delete
    local($tmp, $t_h, $t_b, $t_f); # temp vars
    local($entry);		# a entry from @ready_to_copy
    local($host, $buid, $afile); # host, buid and archive file from an $entry
    local($rem_host, $rem_file); # pieces from $afile
    local($filenumber, $blocknumber); # tape file number
    local($cmd);		# command to execute
    local($result);		# status of command run via system

    $filenumber = 1; $blocknumber = 0;

    # 
    # Orf.  If we are doing things in parallel, then loop around,
    # waiting for backups to finsh, dump them to tape, update the
    # database, etc., until all the children are gone.
    #

main_loop:
    while (1) {
	print "Initializing...\n" if $opt_d & $DEBUG_INFO;

        #
        # If we're copying the stuff from disk to tape, then get a list of the 
        # backups that are ready to be copied, and a count of the backups that 
        # are done or in progress.
        #
        if ($needs_tape) {
	    @ready_to_copy = ();

	    ($in_progress, @ready_to_copy) = &get_buids_for_run($run_id);
	    %hitlist = ();

	    if ($opt_d & $DEBUG_INFO) {
		printf("$in_progress in progress, %d ready to copy\n", 
		       $#ready_to_copy + 1);
		foreach $tmp (@ready_to_copy) {
		    ($t_h, $t_b, $t_f) = split(/\034/, $tmp);
		    print "    $t_h\t$t_b\t$t_f\n";
		}
	    }
	}

	#
 	# If there's nothing to copy, check to see if the kids are 
        # gone.  If they are, we're done.  If they aren't, sleep and
        # loop again.
        #
	if ($#ready_to_copy < 0) {
	    if (kill(0, @pid_list) == 0) {
		print "No kids, none in progress, quitting.\n" if $opt_d & $DEBUG_INFO;
		
		last main_loop;
	    }
		
	    print "None in progress, sleeping...\n" if $opt_d & $DEBUG_INFO;

	    sleep(30);
	    next main_loop;
	}

	#
	# Copy the files that are done to tape...
	#
        if ($needs_tape) {
	    print "Copy to tape...\n" if $opt_d & $DEBUG_INFO;

	    foreach $entry (@ready_to_copy) {
		($host, $buid, $afile) = split(/\034/, $entry);
		($rem_host, $rem_file) = split(/:/, $afile);
	    
		print "Copy host $host, buid $buid, file $afile\n" if $opt_d & $DEBUG_INFO;

		if ($tape_status_ok{$drive_name}) {
		    #
		    # Get the file number on the tape...
		    #
		    if (&tape_position($rsh, $tape_dev, *filenumber, *blocknumber)) {
			print stderr "warning (all-backups): can't copy $buid on $host to tape because\nwe cannot find the tape position.\n";
			&update_budb_for_host($host, $buid, $tape_id, $filenumber, "aborted");
			next;
		    }    

		    if ($filenumber eq "0" || $blocknumber ne "0") {
			print stderr "warning (all-backups): can't copy $buid on $host to tape because the tape is in\nthe wrong position (file $filenumber, block $blocknumber)\n";
			&update_budb_for_host($host, $buid, $tape_id, $filenumber, "aborted");
			next;
		    } 
		}

		#
		# uncompress the file and copy it to the tape...
		#
		$cmd = "$rsh_prog $rem_host -n $zcat_prog  $rem_file | $rsh $dd_prog of=$tape_dev obs=20b conv=block";
		print "all-backups: system '$cmd'\n" if $opt_d & $DEBUG_SYSTEM;
		$result = system($cmd);

		#
		# Update backup db with new TF# and status.  
		#
print "buid $buid tape $tape_id file $filenumber\n";
		&update_budb_for_host($host, $buid, $tape_id, $filenumber, 
				      ($result == 0) ? "done" : "failed", 
				      ($opt_m eq "tape"));
		print stderr "all-backups: command '$cmd' failed!\n"
		  if $result != 0;

		#
		# If this tape drive doesn't do 'mt status' correctly, and 
		# the copy to tape failed, then we really don't know where 
		# the heck we are on the tape now.  lower the accuracy for 
		# subsequent backups to "low".
		#
		if (! $tape_status_ok{$drive_name} && $result != 0) {
		    $tape_host_accuracy = "low";
		}

		#
		# increment the file counter, assuming that this last 
		# copy worked well enough to put something on the tape.
		# This is ignored if we are using 'mt status' to get the 
		# tape position.
		#
		$filenumber++;

		#
		# Add the file to the hit list...
		#
		if ($opt_m eq "tape") {
		    $hitlist{$rem_host} .= "$rem_file ";
		}
	    }
	}


	#
	# For each entry in the hit list, remove the necessary files...
	#
	foreach $rem_host (keys %hitlist) {
	    $cmd = sprintf("$rsh_prog $rem_host -n $rm_prog -f %s",
			   $hitlist{$rem_host});
	    print "all-backups: system '$cmd'\n" if $opt_d & $DEBUG_SYSTEM;
	    system($cmd) && print stderr "warning (all-backups): command '$cmd' failed!\n";
	}
    }

    #
    # Make sure all the children are done...
    #
    while (wait != -1) {
	print "another one bites the dust!\n" if $opt_d & $DEBUG_INFO;
    }

    print "kids are all done!\n" if $opt_d & $DEBUG_INFO;
}


#
# Create an entry in the run database...
#
sub create_run_db {
    local($status, $scheduled) = @_;
    local(%replace, %add, %result);

    %replace = ();
    %replace = ("chain",	$chain,
		"level",	$level,
		"hosts",	$host_string,
		"flags",	$status,
		"active",	(opt_R eq "yes") ? "no" : "yes",
		"start_date",	time);
    if ($status eq "scheduled") {
        $replace{"scheduled_for"} = $scheduled;
    }
    %add = ();

    if (&update_db($run_db_file, $run_db_file, $run_id, "run_db",
		   $DB_CREATE, *replace, *add, *result)) {
	&sigh("error (all-backups): can't create run entry in all-backups!\n");
    }
}

#
# Update the run database for this entry - it is now done.
#

sub update_run_db {
    local(%replace, %add, %result);

    %replace = ();
    $replace{"flags"} = "started";
    $replace{"start_date"} = time;
    %add = ();

    if (&update_db($run_db_file, $run_db_file, $run_id, "run_db",
		   $DB_UPDATE, *replace, *add, *result)) {
        &create_run_db("started", 0);
	# don't worry about the status - it dies if it fails!
    }
}

#
# Update the run database for this entry - it is now done.
#

sub finish_run_db {
    local(%replace, %add, %result);

    %replace = ();
    $replace{"flags"} = "done";
    $replace{"done_date"} = time;
    %add = ();

    if (&update_db($run_db_file, $run_db_file, $run_id, "run_db",
		   $DB_UPDATE, *replace, *add, *result)) {
	&sigh("error (all-backups): huh? no entry for run ID $run_id in main run DB!?\n");
    }
}
