#
# Copyright (c) 1990,1991,1992 The Ohio State University.
# All rights reserved.
#
# Redistribution and use in source and binary forms are permitted
# provided that: (1) source distributions retain this entire copyright
# notice and comment, and (2) distributions including binaries display
# the following acknowledgement:  ``This product includes software
# developed by The Ohio State University and its contributors''
# in the documentation or other materials provided with the distribution
# and in all advertising materials mentioning features or use of this
# software. Neither the name of the University nor the names of its
# contributors may be used to endorse or promote products derived
# from this software without specific prior written permission.
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#

#
# backup-gtar - wrapper for doing backups with GNU tar.  Deals with tar 
# specific things so that do-backups can be more generic.
#

$chain =	0;
$level =	0;
$directory =	".";

require 'global.defs';
require 'local.defs';
require $yagrip_perl;
require $backuplib_perl;
require $lib_deadman_perl if $use_deadman_stuff;

$options = "c:l:f:d:F:Db:m:T:r:";
$usage = 
"usage: backup-gtar [-c chain] [-l level] [-f host:tape] [-d debuglevel]
    [-F files] [-D] [-b buid#] [-T timeout] [-r runid] directory
    -c chain		chain number
    -l level		level number (or name)
    -f host:tape	host and tape to use
    -d debuglevel	print debugging messages
    -F files		where to log dump directory (NYI)
    -D			we're archiving to disk, not tape
    -b buid#		backup id number for this backup
    -T timeout		timeout for this backup
    -r runid		run ID for this run
    directory		which directory to back up\n";

&init();

#
# Invoke the deadman watcher...
#
&deadman($timeout, $opt_r, "$hostname.r$opt_r", "$hostname.b$buid") 
  if $use_deadman_stuff;

    #
    # Get the info about the latest lower level dump from this dumpdates 
    # file. 
    #
%backup_list = ();
%backup_times = ();
&get_recent_backups($hostname, $directory, time, *backup_list, *backup_times);
@backup_keys = sort chrono keys %backup_list;

&open_db(*RUNDB, $run_db_file, $db_mode) ||
  die "error (backup-gtar): can't open the dbm file '$run_db_file'\n";
($latest_run, $latest_chain, $last_level, $latest_backup, $last_timestamp) =
  &get_latest_lower_or_same_level($chain, $level, *backup_list, *backup_keys);
chop($last_date = &ctime($last_timestamp));
&close_db(*RUNDB, $run_db_file);

print "backup-gtar: last level $last_level, date $last_date, ts $last_timestamp\n" 
  if $opt_d & $DEBUG_INFO;

if ($use_chain_files &&
    ! &check_funky_file($directory, $chain, $level, $last_level, $last_timestamp)) {
    exit(1);
}

$now_date = &getdate();
$now_timestamp = time;

$volhdr = "$hostname $directory $chain $level $now_date";

#
# Update from_date, to_date in this hosts's BU DB
#
print "backup-gtar: update dates in per host backup DB\n" if $opt_d & $DEBUG_INFO;

%add = ();
%replace = ();
%replace = ("fromdate",	$last_timestamp,
	    "volhdr",	$volhdr,
	    "todate",	$now_timestamp);
if (&update_db($backup_host_file . $hostname, $backup_host_file . $hostname,
	       $buid,"backup_host", $DB_UPDATE, *replace, *add, *result)) {
    &sigh("$0: cannot update DB entry for $buid in backup host file for $hostname\n");
}

    #
    # If we are in parallel mode, save STDOUT, close it and reopen it to 
    # shove the archive through compress and into the archive file.
    #
if (defined($opt_D)) {
    open(SAVEOUT, ">&STDOUT");	# save STDOUT, 
    close(STDOUT);

    $cmd = "| $compress_prog > $opt_f.Z";
    print stderr "backup-gtar: open '$cmd'\n" if $opt_d & $DEBUG_SYSTEM;

    if (! open(STDOUT, $cmd)) {
	&clean_up_dd();
	die "error (backup-gtar): can't pipe archive to compress";
    }
}

    #
    # Prepare the command line for running GNU tar.
    #
if (defined($opt_D)) {
    $archive_file = "-";
} else {
    ($tape_user, $tape_host, $tape_dev) = &splittapedev($opt_f);
    if ($tape_host eq $hostname) {
	$archive_file = $tape_dev;
    } else {
	if ($tape_user ne "" && $gtar_does_remote_users) {
	    $archive_file = "$tape_user@$tape_host:$tape_dev";
	} else {
	    $archive_file = "$tape_host:$tape_dev";
	}
    }
}

    #
    # How to deal with the "file" list with GNU tar...
    #
undef $opt_F;			# ignore -F for now. (fixme)
if (defined($opt_F)) {
    $toc_flag = "v";
    $toc_file = $opt_F;
} else {
    $toc_flag = "";
    $toc_file = "";
}

chdir($directory) || die "error (backup-gtar): can't chdir to $directory\n";

if ($level == 0) {
    $incremental = "";
} else {
    $incremental = "-G -N \"$last_date\"";
}

if ($gtar_listed_incrementals) {
    if ($level != 0) {
	if ($latest_backup != -1) {
	    $old_incr_db_logfile = "$hostname.$latest_backup";
	} else {
	    die "error (backup-gtar): you need to do a level 0 backup before any incrementals!\n";
	}
    }

    # Create the tar_db_dir if it doesn't exist.

    if (! -d $tar_db_dir) {
	mkdir($tar_db_dir, $default_dir_mode) || 
	  die "error (backup-gtar): cannot mkdir $tar_db_dir ($!)\n";
    }

    # Set the name of the current run's tar database file.

    $incr_db_logfile = "$hostname.$buid";
    $incremental = " --listed-incremental $tar_db_dir/$incr_db_logfile ";

    #
    # If this is not a level 0 dump, copy the previous dump's database file
    # to the current dump's database file.
    #
    if ( $level != 0 ) {
	open(OLD_TAR_DB, "< $tar_db_dir/$old_incr_db_logfile") || 
	  die "error (backup-gtar): can't open old tar db file $old_incr_db_logfile\n";

	open(NEW_TAR_DB, "> $tar_db_dir/$incr_db_logfile") || 
	  die "error (backup-gtar): can't open new tar db file $incr_db_logfile\n";

	while ( <OLD_TAR_DB> ) {
	    print NEW_TAR_DB;
	}

	close(OLD_TAR_DB);
	close(NEW_TAR_DB);
    }
}

    #
    # Give the user a chance to change the exclude file in a funky way.
    #
&gtar_exclude_hook();

$command = "$gtar_prog -c -S -f $archive_file $gtar_extra_args $gtar_vol_prog -M -l -V \"$volhdr\" $gtar_exclude $incremental .";

print stderr "backup-gtar: system $command\n" if $opt_d & $DEBUG_SYSTEM;

$result = system($command);

if ($result != 0) {
    printf(stderr "error (backup-gtar): GNU tar failed with status %d (%d)\n", 
	   $result, 
	   int($result/256));
    exit(1);
}

    #
    # If we are in parallel mode, redirect STDOUT back to what it 
    # was before.
    #
if (defined($opt_D)) {
    close(STDOUT);
    open(STDOUT, ">&SAVEOUT");
}

exit(0);


#
# Deal with command line args and do init stuff
#
sub init {

    #
    # Make sure that we clean up after ourselves if someone wants to kill us.
    #
    $SIG{'INT'} = 'handle_interrupts';
    $SIG{'HUP'} = 'handle_interrupts';
    $SIG{'TERM'} = 'handle_interrupts';
    $SIG{'QUIT'} = 'handle_interrupts';

    #
    # deal with args
    #	default for tapehost and tapedev is scarecrow and /dev/rst1 for now
    #
    &read_config();		# Read backup.config file

    &getopt($options) || die $usage;

    die $usage if $#ARGV != 0;

    if (! defined($opt_c) || ! defined($opt_l) || ! defined($opt_f) 
	|| ! defined($opt_b) || ! defined($opt_r)) {
	die "error (backup-gtar): -c, -l, -f, -b and -r are required options (oxymoron?)\n";
    }

    $directory = $ARGV[0];
    $chain = $opt_c;
    $level = $opt_l;
    $buid = $opt_b;
    $ENV{"BUID"} = $buid;

    if (defined($opt_T)) {
	$timeout = $opt_T;
    } else {
	$timeout = 0;
    }
}

