#
# Copyright (c) 1991,1992 The Ohio State University.
# All rights reserved.
#
# Redistribution and use in source and binary forms are permitted
# provided that: (1) source distributions retain this entire copyright
# notice and comment, and (2) distributions including binaries display
# the following acknowledgement:  ``This product includes software
# developed by The Ohio State University and its contributors''
# in the documentation or other materials provided with the distribution
# and in all advertising materials mentioning features or use of this
# software. Neither the name of the University nor the names of its
# contributors may be used to endorse or promote products derived
# from this software without specific prior written permission.
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#

$FontSize = 10;
$LineHeight = $FontSize * 1.2;
$Top = 720;
$Left = 72;
$Middle = 204;
$Right = 576;
$Bottom = 72;
$Border = 2;

$Page_Count = 0;

$options = "d:hN:T:";
$usage = "usage: backup-report [-p pagesize] [-N daysago] [-a] 
  [-T date] [-d debuglevel] [-h] [host[:filesystem]]
    -N daysago			restrict listing to backups within
				this many days
    -T date                     show only backups before that date
    -d debuglevel		display debugging messages
    -h				display this message
    host:filesystem             restrict display to just this host
                                (and file system if given).\n";

require 'global.defs';
require 'local.defs';
require $yagrip_perl; 
require $backuplib_perl;
require $unctime_perl;
require $ctime_perl;

    #
    # Make sure that we clean up after ourselves if someone wants to kill us.
    #
$SIG{'INT'} = 'handle_interrupts';
$SIG{'HUP'} = 'handle_interrupts';
$SIG{'TERM'} = 'handle_interrupts';
$SIG{'QUIT'} = 'handle_interrupts';

    # 
    # Deal with args...we define *_string things to use as args for progs 
    # called from here so that we can pass on the right values (eg, -d
    # or "", etc). 
    #
die $usage if ! &getopt($options);
die $usage if defined($opt_h);

if (defined($opt_T)) {
    $upper_time = &parse_date_into_timestamp($opt_T);
} else {
    $upper_time = time;
}

if (defined($opt_N)) {
    $lower_time = $upper_time - $opt_N * $DAYS;
} else {
    $lower_time = 0;
}

    #
    # Read the backup config file...
    #
&read_config();

    #
    # We need the run database to get the chain, level numbers...
    #
&open_db(*RUNDB, $run_db_file, $db_mode) ||
  &sigh("clean-backup-report: can't open the dbm file '$run_db_file'\n");

    #
    # open each of the per-host backup database files in turn...
    #
if ($#ARGV == -1) {
    @bu_host_files = <$backup_host_file*.$dbext>;
} else {
    while ($#ARGV > -1) {
	($host, $fsys) = split(/:/, shift(@ARGV));
	push(@bu_host_files, "$backup_host_file$host.$dbext");
	if ($fsys ne "") {
	    if (defined($file_sys_limits{$host})) {
		$file_sys_limits{$host} .= " $fsys";
	    } else {
		$file_sys_limits{$host} = $fsys;
	    }
	}
    }
}

    #
    # Write out the boilerplate...
    #
&BeginBoiler();
&StartPage();

foreach $budb_name (@bu_host_files) {
    # reset the arrays...
    %fs_list = (); %chain = (); %level = (); %status = (); %backup_times = (); %fs_name = ();

    #
    # Get the corresponding fsys limit
    #
    $fsys_limit = shift(@file_sys_limit);

    #
    # Need to nuke the trailing .$dbext to get the base db name, 
    # and get the host name for other uses.
    #
    @tmp = split(/\./, $budb_name);
    pop(@tmp);			# nuke trailing .$dbext
    $thishost = $tmp[$#tmp];	# host name is last before .$dbext
    $budb_name = join('.', @tmp);

    &open_db(*LIST, $budb_name, $db_mode) ||
       &sigh("clean-db: can't open the dbm file '$budb_name'\n");

    #
    # For each backup in this database...read it, cache the printable
    # copy, and store the backup ID in the appropriate list for that 
    # file system.
    #
  bu_entry_loop:
    foreach $buid (&keys_db(*LIST)) {
	# no value - bogus entry, nuke it.
	next bu_entry_loop if ! &read_db(*LIST, $buid, *budb_value, *budb_array); 

	#
	# is it a keeper?  Only if done between lower time limit and upper time limit.
	#
	if ($budb_array{"todate"}+0 < $lower_time ||
	    $budb_array{"todate"}+0 > $upper_time) {
	    next bu_entry_loop;
	}

        #
        # get the chain and level from the run database
        #
	$run = $budb_array{"run"};
	$fs = $budb_array{"fs"};

	if (! defined($run_chain{$run})) {
	    &read_db(*RUNDB, $run, *value, *array);
	    $run_chain{$run} = $array{"chain"};
	    $run_level{$run} = $array{"level"};
	}

	$chain{$buid} = $run_chain{$run};
	$level{$buid} = $run_level{$run};

	#
	# Ignore entries that have been reused or which failed...
	#
	if (((! defined($budb_array{"flags"}) || $budb_array{"flags"} ne "done") &&
	     (! defined($budb_array{"d-flags"}) || $budb_array{"d-flags"} ne "done"))) {
	    $status{$buid} = 0;
	} else {
	    $status{$buid} = 1;
	}

        #
        # If we don't know about this file system, add it to %fs_list and 
        # make a name for its list of backup IDs.
        #
	if (! defined($fs_list{$fs})) {
	    $fs_list{$fs} = 1;
	    $fs_name{$fs} = sprintf("a%d", $count++);
	}

        #
        # store the todate time in the %backup_times array for quicker sorting
        #
	$backup_times{$buid} = $budb_array{"todate"} + 0;

        #
        # and store the backup ID in the appropriate array for this file 
        # system.
        #
	eval(sprintf("push(@%s, $buid)", $fs_name{$fs}));
	die "error (backup-report): $@" if $@;
    }

    #
    # ok, we've read through all of the backup entries and sorted them 
    # according to file system.  for each file system, in alphabetical 
    # order, sort the backup IDs into chrono order and print them...
    #
    if (defined($file_sys_limits{$thishost})) {
	@file_systems = split(/ /, $file_sys_limits{$thishost});
    } else {
	@file_systems = sort keys %fs_list;
    }

    foreach $fs (@file_systems) {
        if ($Ypos < $Bottom) {
	    &EndPage();
	    &StartPage();
	}

	print "($thishost:$fs) $Left $Ypos ShowFS\n";

	@backups_for_fs = eval(sprintf("@%s", $fs_name{$fs}));
	die "error (backup-report): $@" if $@;

	print "GlyphFont\n";

        foreach $buid (sort chrono @backups_for_fs) {
	    $Xpos = &time_to_xpos($backup_times{$buid});
	    printf "(%d) $Xpos $Ypos %d ShowGlyph\n", $level{$buid}, 
	           $status{$buid};
	}

	&NewLine();

	# free the memory used by the list of backups...
	eval(sprintf("@%s = ()", $fs_name{$fs}));
	die "error (backup-report): $@" if $@;
    }

    &close_db(*LIST, $budb_name);
}

&close_db(*RUNDB, $run_db_file);

if ($Ypos < $Top - $LineHeight) {
    &EndPage();
}

&EndBoiler();

exit(0);


sub BeginBoiler {

    print "%!PS-Adobe-2.0
%%Pages: (atend)
%%BeginProlog
/TextFont {
    /Times-Roman findfont $FontSize scalefont setfont
} def

/GlyphFont {
    /Courier findfont $FontSize scalefont setfont
} def

0 0 moveto
GlyphFont (0) false charpath flattenpath pathbbox    % llx lly urx ury
    2 index sub $Border dup add add /ywidth exch def    % llx lly urx
    2 index sub $Border dup add add /xwidth exch def    % llx lly
    pop pop

/MakeBox {			% x y ==> 
    newpath
    moveto
    $Border neg $Border neg rmoveto
    0 ywidth rlineto
    xwidth 0 rlineto
    0 ywidth neg rlineto
    xwidth neg 0 rlineto
    closepath
    0 setgray
    fill
} def

/ShowGlyph {
    /status exch def
    /ypos exch def
    /xpos exch def
    /level exch def
    status 0 eq { xpos ypos MakeBox } if
    xpos ypos moveto
    status 0 eq { 1 setgray } { 0 setgray} ifelse
    level show
} def

/ShowFS {			% text x y
    moveto
    TextFont
    0 setgray 
    show
} def
	
/StartPage {
    .5 setgray
    newpath $Middle $Top moveto $Right $Top lineto stroke
    newpath $Middle $Top moveto $Middle $Bottom lineto stroke
    newpath $Right $Top moveto $Right $Bottom lineto stroke
    newpath $Middle $Bottom moveto $Right $Bottom lineto stroke
";

    @local_time = localtime($upper_time);
    $start_of_day = $upper_time - 
      $local_time[0] - $local_time[1] * 60 - $local_time[2] * 60 * 60;

    for (;$start_of_day > $lower_time; $start_of_day -= $DAYS) {
	$day_pos = &time_to_xpos($start_of_day);
	print "    newpath $day_pos $Top moveto $day_pos $Bottom lineto stroke\n";
    }

    print "	
} def

%%EndProlog
";
};

sub EndBoiler {
    print "
%%Trailer
%%Pages: $Page_Count
%%EOF\n";
};

sub NewLine {
    $Ypos = $Ypos - $LineHeight;
};

sub StartPage {
    $Page_Count++;

    print "%%Page: $Page_Count $Page_Count
%%BeginPageSetup
/pgsave save def
%%EndPageSetup

StartPage\n";
    $Ypos = $Top - $LineHeight;
}

sub EndPage {
    print "pgsave restore
showpage\n";
}

sub time_to_xpos {
    local($time) = @_;
    local($ratio, $position);

    $ratio = ($time - $lower_time) / ($upper_time - $lower_time);
    $position = $Middle + $ratio * ($Right - $Middle);
    return($position);
}

