#
# Copyright (c) 1990,1991,1992 The Ohio State University.
# All rights reserved.
#
# Redistribution and use in source and binary forms are permitted
# provided that: (1) source distributions retain this entire copyright
# notice and comment, and (2) distributions including binaries display
# the following acknowledgement:  ``This product includes software
# developed by The Ohio State University and its contributors''
# in the documentation or other materials provided with the distribution
# and in all advertising materials mentioning features or use of this
# software. Neither the name of the University nor the names of its
# contributors may be used to endorse or promote products derived
# from this software without specific prior written permission.
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#

#
# Real simple - figure out what backups need to be done for the
# directories that should be backed up on this host.  Details
# explained below.  By default, message is printed to stdout.  The -m
# option means "mail it" to the $mail_to folks.  The -n option means 
# don't mean unless there's a problem, and is only useful if -m is also 
# given. 
#

$options = "hmnd:";
$usage = 
"usage: check-backups [-m] [-n] [-d debuglevel] [-h] [host]
    -m			send results by mail to '%s'
    -n			mail results only if there are problems
    -f debuglevel 	display debugging output
    -h			display this message
    host		name of the host to check.\n";

require 'global.defs';
require 'local.defs';
require $yagrip_perl;
require $backuplib_perl;

    #
    # Make sure that we clean up after ourselves if someone wants to kill us.
    #
$SIG{'INT'} = 'handle_interrupts';
$SIG{'HUP'} = 'handle_interrupts';
$SIG{'TERM'} = 'handle_interrupts';
$SIG{'QUIT'} = 'handle_interrupts';

&read_config();

if ($config_name ne "") {
    $id_label = "[$config_name] ";
    $insert = ".$config_name.";
} else {
    $id_label = "";
    $insert = "";
}

# 
# Deal with args...
#
die $usage if ! &getopt($options);
die $usage if defined($opt_h);

#
# If there's an arg, its the name of the host to check.
#
if ($#ARGV == 0) {
    $hostname = shift;
    $doing_some_other_host = 1;
}

die $usage if $#ARGV != -1;

#
# $bitch is a list of reasons why the backups need to be done
# %need_to_do is a list of the backups that need to be done
#
$bitch = "";
%need_to_do = ();

if (! $doing_some_other_host && open(FSTAB, $fstab_file_name)) {
    while (<FSTAB>) {
	chop;
	($line) = split(/\#/);
	($device, $directory, $type) = &read_fstab_entry($line);
	if (&found_in_list($type, @fs_types)) {
	    $fs_check_list{$directory} = 1;
	}
    }

    close(FSTAB);
}

#
# The fun part...for each filesystem, chain and level combination,
# make sure that the (chain, level) backup for that filesystem was done
# within EVERY of the preceeding backup.  
#

open(BACKUP, $backup_file) || die "error (check-backups): can't open $backup_file";

#
# For each directory in /etc/backups...
#
backup_loop:
while (&read_backups_entry(*BACKUP, *host, *directory, *archive_type, *options, *tags, *chains)) {

    next backup_loop if $hostname ne $host;

    print "check-backups: work on $directory\n" if $opt_d & $DEBUG_INFO;

    #
    # fs_check_list is a list of the file systems that haven't been seen 
    # in /etc/backups yet.
    #
    if (! $doing_some_other_host && defined($fs_check_list{$directory})) {
	delete $fs_check_list{$directory};
    }

    next if $archive_type eq "none";

    #
    # Get the list of recent backups for this directory.
    #
    %backup_list = ();
    %backup_times = ();
    &get_recent_backups($hostname, $directory, time, *backup_list, *backup_times);
    @backup_keys = sort chrono keys %backup_list;

    #
    # For each chain...
    #
    @chains_to_check = keys %chains;
    $chain_count = $#chains_to_check;

    if ($#chains_to_check == -1) {
	@chains_to_check = 0..($num_chains - 1);
    }

    &open_db(*RUNDB, $run_db_file, $db_mode) ||
      &sigh("check-backups: can't open the dbm file '$run_db_file'\n");

  chain_loop:
    foreach $m_chain (@chains_to_check) {
	#
	# If there was a chain= entry for this chain, then get 
	# the max level to check for this chain.  -1 means no limit - 
        # check them all.  
	#
	if (defined($chains{$m_chain})) {
	    $m_level = $chains{$m_chain} + 0;
	    if ($m_level == -1) {
		$m_level = $max_level[$m_chain] + 0;
	    }
	} 
	#
	# Otherwise, if there were other chain entries, we can 
	# skip this chain.  If there were no other chain entries, 
	# we must be doing all levels of all chains.
	#
	else {
	    if ($chain_count >= 0) {
		next chain_loop;
	    } else {
		$m_level = $max_level[$m_chain] + 0;
	    }
	}

	#
	# Check to see that the chain is a valid chain number...
	#
	if ($m_chain >= $num_chains) {
	    $bitch .= 
	      sprintf("$backup_file entry for %s contains an invalid chain: %d\n", 
			  $directory,
			  $m_chain);
		next chain_loop;
	}

        if ($use_chain_files && ! $doing_some_other_host) {
	    #
	    # First, check the .chain file to see if its missing (in which
	    # case we assume that a full save should be done) or newer
	    # than the latest full save (indicating that a restore was
	    # done and that a full save should be done).
	    #
	    if ($directory eq "/") {
		$name = "/.chain$insert$m_chain";
	    } else {
		$name = "$directory/.chain$insert$m_chain";
	    }

	    if (-f $name && (@stats = stat(_))) {
		($latest_run, $latest_chain, $latest_level, $latest_backup, 
		 $latest_todate) =
		  &get_latest_lower_or_same_level($m_chain, 0, *backup_list, *backup_keys);

		print "run $latest_run chain $latest_chain level $latest_level backup $latest_backup date $latest_todate\n" if $opt_d & $DEBUG_INFO;

		if ($latest_backup == -1) {
		    $then = 0;
		} else {
		    $then = $latest_todate + 0;
		}

		if ($stats[10]+0 > $then) {
		    $bitch .= 
"$directory needs a $m_chain,0 backup -- looks like a full restore was done
    (or a level 0 dump failed)\n";
		    $need_to_do{"chain $m_chain level 0"} = 1;
		    next chain_loop;
		}
	    } else {
		$bitch .= 
"$directory needs a $m_chain,0 backup -- the .chain file is missing\n";
		$need_to_do{"chain $m_chain level 0"} = 1;
		next chain_loop;
	    }
	}

	#
	# Ok, for each level that we are supposed to check...
	#
	@levels_to_check = split(/ /, $level_number_list{$m_chain});
	for ($i = 0; $i < $#levels_to_check; $i++) {
	    $levels_to_check[$i] += 0; # convert to integers
	}
	
      level_loop:
	foreach $level (sort @levels_to_check) {
	    # 
	    # don't need to check any more than the limit given in the 
	    # backups file (if there is one).
	    #
	    if ($level+0 > $m_level) {
		last level_loop;
	    }

	    #
	    # How often this level should be done...
	    #
	    $every = $level_every{$m_chain, $level} + 0;

            ($latest_run, $latest_chain, $latest_level, $latest_backup, $latest_todate) =
	      &get_latest_lower_or_same_level($m_chain, $level, *backup_list, *backup_keys);

	    print "latest backup of $directory before $m_chain, $level:\n" 
	      if $opt_d & $DEBUG_INFO;
	    print "  run $latest_run chain $latest_chain, $latest_level backup $latest_backup date $latest_todate\n"
	      if $opt_d & $DEBUG_INFO;

	    if ($latest_backup == -1) {
		$then = 0;
	    } else {
		$then = $latest_todate + 0;
	    }

	    printf "chain %d, %d time elapsed is %d, every is %d\n",
	           $m_chain, $level, time - $then, $every
	      if $opt_d & $DEBUG_INFO;

	    if (time - $then >= $every) {
		    $bitch .= 
		      sprintf("%s is overdue for a %d,%d backup (%.1f days)\n",
			      $directory,
			      $m_chain,
			      $level,
			      (time - $then) / ($DAYS));
		    $need_to_do{"chain $m_chain level $level"} = 1;
		    last level_loop;
            } # if there's a problem
	} # foreach level
    } # foreach chain

    &close_db(*RUNDB, $run_db_file);
} # foreach entry in BACKUP

close(BACKUP);

if (! $doing_some_other_host) {
    $unchecked_filesystems = join(' ', keys %fs_check_list);
    if ($unchecked_filesystems ne "") {
	$bitch .= "\nThese file systems are in /etc/fstab or /etc/checklist 
but not in $backup_file: $unchecked_filesystems\n";
    }
}

    #
    # If everything was ok (bitch list is empty)...all is well...
    #
if ($bitch eq "") {
    if (defined($opt_m) && ! defined($opt_n)) {
	$cmd = sprintf("$mail_prog -s \"%scheck-backups on %s - everything's ok!\" %s",
		       $id_label,
		       $hostname,
		       $mail_to);
	system($cmd);
    }
} else {

    #
    # If bitch list isn't empty, construct the bitch message...
    #
    $foo = "The following backups need to be done:\n";

    foreach $backup (keys %need_to_do) {
	$foo .= sprintf("\t%s\n", $backup);
    }
    $foo .= "\n";
    $bitch = $foo . $bitch;

    if (defined($opt_m)) {
        $cmd = sprintf("| $mail_prog -s \"%scheck-backups on %s - problems!\" %s",
		       $id_label,
		       $hostname,
		       $mail_to);

        open(MAIL, $cmd) ||
          die "error (check-backups): can't send mail";
        printf(MAIL $bitch);
        close(MAIL);
    } else {
        printf($bitch);
    }
}

