#
# Copyright (c) 1990,1991,1992 The Ohio State University.
# All rights reserved.
#
# Redistribution and use in source and binary forms are permitted
# provided that: (1) source distributions retain this entire copyright
# notice and comment, and (2) distributions including binaries display
# the following acknowledgement:  ``This product includes software
# developed by The Ohio State University and its contributors''
# in the documentation or other materials provided with the distribution
# and in all advertising materials mentioning features or use of this
# software. Neither the name of the University nor the names of its
# contributors may be used to endorse or promote products derived
# from this software without specific prior written permission.
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#

$chain =	0;
$level =	0;

$options =	"c:l:f:d:t:r:px:i:m:";
$usage = 
"usage: do-backups [-c chain] [-l level] [-f host:dev] [-d debuglevel] [-t tapeid]
    [-r runid#] [-p] [-i tag] [-x tag] [-m tape|disk|both]
    -c chain		chain number
    -l level		level number (or name)
    -f host:tape	host and tape to use
    -t tape#		tape number that should be loaded in the drive
    -d debuglevel	print debugging messages
    -p			we're running in parallel mode
    -b buid#		backup id number for this backup
    -r runid#		run id number for this run
    -m tape|disk|both	store the archives on tape, disk or both
    -i tag		include only file systems with this tag
    -x tag		exclude file systems with this tag\n";

require 'global.defs';
require 'local.defs';
require $yagrip_perl;
require $backuplib_perl;
require $lib_deadman_perl if $use_deadman_stuff;

&init();

#
# invoke the deadman watcher, no timeout.  Die if the switch for the 
# run appears, or if HOST.rRUNID appears.
#
&deadman(0, $opt_r, "$hostname.r$opt_r") if $use_deadman_stuff;

    #
    # Create the entry in this hosts's run DB.  Yes, its empty.
    #
%replace = ();
%add = ();
if (&update_db($run_host_file . $hostname, $run_host_file . $hostname,
	       $opt_r, "run_host", $DB_CREATE, *replace, *add, *result)) {
    &sigh("error (do-backups): cannot create entry for $opt_r in run host DB on $hostname\n");
}

    #
    # Read through $backup_file and figure out what backups to do...
    #
open(BACKUP, $backup_file) || &sigh("error (do-backups): can't open $backup_file");

backup_loop:
while (&read_backups_entry(*BACKUP, *host, *directory, *archive_type, *other, *tags, *chains)) {

    next if $host ne $hostname;

    print "do-backups: work on $directory\n" if $opt_d & $DEBUG_INFO;

    #
    # If we should skip this backup, go on to the next backup file entry...
    #
    next backup_loop if &should_skip_this_backup();

    warn "[backing up $hostname:$directory]\n";
    $backups_done++;
    $ENV{"BU_DIRECTORY"} = $directory;	# for use by extra scripts, in particular.
    $ENV{"BU_HOST"} = $host;
    #
    # Everyone needs a backup id number - lets get ours, they're free...
    #
    $buid = &get_id($backup_host_file . $hostname);

    #
    # where's the archive going to go?  If we are doing a parallel backup, 
    # or storing it online, then use the $online directory.  Otherwise,
    # use the (hopefully) $opt_f tape device.
    #
    if ($to_disk || defined($opt_p)) {
	if ($other{"online"} eq "") {
	    warn "warning (do-backups): you haven't defined where you want to store the backup!\nI'm skipping this file system.\n";
	    next backup_loop;
	}

	($archive_host, $archive_file) = split(/:/, $other{"online"});
	if ($archive_file eq "") {
	    $archive_file = $archive_host;
	    $archive_host = $hostname;
	}
	$archive_file = "$archive_file/$hostname.$buid";
	$doing_online = 1;
    } else {
	if (! defined($opt_f)) {
	    warn 
"warning (do-backups): you didn't give the -f option to specify where the tape was!  Skipping to the next backup...\n";
	    next backup_loop;
	}

	$archive_file = $tape_user_host_dev;
	$doing_online = 0;
    }

    if (defined($opt_t) && ! defined($opt_p)) {
	if ($status_ok) {
	    #
	    # This tape host can deal with 'mt status' correctly.
	    #
	    if (&tape_position($rsh, $tape_dev, *file_number, *block_number)) {
		warn "warning (do-backups): can't get tape position, skipping to next backup.\n";
		next backup_loop;
	    }

	    if ($block_number ne "0") {
		warn "warning (do-backups): tape position error, not at block 0 (file $file_number, block $block_number)\n";
		next backup_loop;
	    }
	    $accuracy = "high";
	} else {
	    #
	    # This tape host cannot deal with 'mt status' correctly.  Get 
	    # the estimated file number and confidence from the tape database.
	    #
	    if (&update_db($tape_db_file, $tape_db_file, $opt_t, "tape_db", 
			   $DB_READ, *replace, *add, *result)) {
	        &sigh("error ($0): there's no DB entry for tape $opt_t.\n");
	    }

	    $file_number = $result{"file_number"};
	    $accuracy = $result{"accuracy"};
	}
    }

    #
    # Update the entry in this hosts's run DB
    #
    %replace = ();
    %add = ();
    $add{"buid_list"} = $buid;
    if (&update_db($run_host_file . $hostname, 
		   $run_host_file . $hostname,
		   $opt_r, "run_host", $DB_UPDATE,
		   *replace, *add, *result)) {
        &sigh("error ($0): cannot update entry for $opt_r in run host DB on $hostname\n");
    }

    #
    # Add this BUID to this hosts's BU DB
    #
    print "do-backups: update per host backup DB\n" if $opt_d & $DEBUG_INFO;

    %replace = ();
    %replace = ("run",		$opt_r,
		"fs",		$directory,
		"volhdr",	"",
		"type",		$archive_type);

    if ($doing_online) {
	$replace{"d-flags"} = "started";
	$replace{"afile"}  = "$archive_host:$archive_file.Z";
    } else {
	if ($file_number == 0) {
	    $replace{"flags"} = "aborted";
	} else {
	    $replace{"flags"} = "started";
	}
    }

    if ($to_tape) {
	$replace{"tflist"} = "$opt_t.$file_number";
	$replace{"accuracy"} = $accuracy;
    }
    %add = ();

    if (&update_db($backup_host_file . $hostname, $backup_host_file . $hostname,
		   $buid,"backup_host", $DB_CREATE, *replace, *add, *result)) {
        &sigh("error ($0): cannot create DB entry for $buid in backup host file for $hostname\n");
    }

    #
    # No reason to continue if we're only going to tape, and not disk at all, 
    # and we're at file number 0...
    #
    if ($to_tape && ! $to_disk && ! defined($opt_p) && $file_number == 0) {
	&sigh("error (do-backups): YOW, we're at BOT, aborting this backup.\n");
    }

    #
    # Create the command to be executed...
    #

    $io_redirection = "-F $file_dir/$hostname.$buid";

    if ($other{"timeout"} ne "") {
	$timeout_string = "-T $other{'timeout'}";
    } else {
	$timeout_string = "";
    }

    $command = "$backup_prog-$archive_type $run_string -c $chain -l $level -f $archive_file $debug_string $io_redirection $disk_string $media_string -b $buid $timeout_string $directory";
    $error = 0;

    print "do_backups: $command\n" if $opt_d & $DEBUG_SYSTEM;

    if (! -d $error_dir) {
	mkdir($error_dir, $default_dir_mode) ||
	  warn "warning (do-backups): can't make directory $error_dir ($!)\n";
    }

    open(NEWOUT, "| $tee_prog $error_dir/$hostname.$buid") ||
      &sigh("error (do-backups): can't pipe results through tee");
    open(SAVEOUT, ">&STDOUT"); # save STDOUT
    open(SAVEERR, ">&STDERR"); # save STDERR

    close(STDOUT);
    open(STDOUT, ">&NEWOUT");
    $oldfh = select(STDOUT); $| = 1; select($oldfh);

    close(STDERR);
    open(STDERR, ">&NEWOUT");
    $oldfh = select(STDERR); $| = 1; select($oldfh);

    if ($other{"script"} ne "") {
	$cmd = "$other{'script'} stop";
	print "do-backups: running '$cmd'\n" if $opt_d && $DEBUG_SYSTEM;
	if (($system_result = system($cmd)) != 0) {
	    printf stderr "warning (do-backups): '$cmd' failed with status %d (errno %d)\n",
	        $system_result >> 8, $system_result & 0xff;
	    $error = 1;
	}
    }	

    $result = system($command);

    if ($other{"script"} ne "") {
	$cmd = "$other{'script'} start";
	print "do-backups: running '$cmd'\n" if $opt_d && $DEBUG_SYSTEM;
	if (($system_result = system($cmd)) != 0) {
	    printf stderr "warning (do-backups): '$cmd' failed with status %d (errno %d)\n",
	        $system_result >> 8, $system_result & 0xff;
	    $error = 1;
	}
    }	

    close(STDOUT);
    open(STDOUT, ">&SAVEOUT");
    close(STDERR);
    open(STDERR, ">&SAVEERR");

    #
    # Deal with the results...
    #
    if ($result != 0) {
	warn "warning (do_backups): command failed with status $result\n";
	$status = "failed";
	if ($to_tape && ! $to_disk && ! defined($opt_p) && ! $status_ok) {
	    # backup to tape failed, tape host doesn't deal with 'mt status', 
	    # accuracy of file position is now low.
	    $accuracy = "low";
	}
    } else {
	$status = "done";
    }

    if ($doing_online) {
	&update_buid_flag("d-flags", $status);
    } else {
	&update_buid_flag("flags", $status);
    }

    #
    # If we are doing on line, and it needs to be copied to tape, 
    # and not parallel, copy it...
    #
    if ($doing_online && $to_tape && ! defined($opt_p) && $file_number != 0) {
	&update_buid_flag("flags", "started");

	$cmd = "$zcat_prog $archive_file | $rsh $dd_prog of=$tape_dev obs=20b conv=block";
	print "do-backups: system '$cmd'\n" if $opt_d & $DEBUG_SYSTEM;

	$result = system($cmd);
        if ($result == 0) {
	    $status = "done";
	} else {
	    $status = "failed";
	    $result = $result/256;
	    warn "warning (do-backups): command '$cmd' failed ($result)!\n";
	    if (! $status_ok) {
		$accuracy = "low";
	    }
	}

	&update_buid_flag("flags", $status);
    }

    if ($to_tape && ! defined($opt_p)) {    
	$file_number++;
	%replace = ();
	$replace{"file_number"} = $file_number;
	$replace{"accuracy"} = $accuracy;
	%add = ();
	if (&update_db($tape_db_file, $tape_db_file, $opt_t, "tape_db", $DB_UPDATE, 
		       *replace, *add, *result)) {
	    &sigh("error ($0): cannot update entry for tape $opt_t in the tape DB - no such entry.\n");
	}
    }
}

close(BACKUP);

print "warning (do-backups): no backups to do on $hostname?!\n" 
  if $backups_done == 0;

exit(0);


sub init {
    #
    # Make sure that we clean up after ourselves if someone wants to kill us.
    #
    $SIG{'INT'} = 'handle_interrupts';
    $SIG{'HUP'} = 'handle_interrupts';
    $SIG{'TERM'} = 'handle_interrupts';
    $SIG{'QUIT'} = 'handle_interrupts';

    # 
    # Deal with args...
    #
    &read_config();		# Read backup.config file

    &getopt($options) || die $usage;

    die $usage if $#ARGV != -1;

    if (! defined($opt_c) || ! defined($opt_l) || ! defined($opt_r)) {
	die "error (do-backups): -c, -l and -r are required arguments.\n$usage";
    }

    $chain = $opt_c + 0;
    $level = $opt_l + 0;

    if ($opt_m ne "tape" && $opt_m ne "disk" && $opt_m ne "both") {
	die "error (do-backups): media type $opt_m isn't one of tape, disk or both.\n$usage";
    }
    $media_string = "-m $opt_m";

    $to_tape = 1 if $opt_m eq "tape" || $opt_m eq "both";
    $to_disk = 1 if $opt_m eq "disk" || $opt_m eq "both";

    if (defined($opt_p) || $to_disk) {
	$disk_string = "-D";
    } else {
	$disk_string = "";
    }

    if ($to_tape && ! defined($opt_f)) {
	die "error (do-backups): backing up to tape, but no tape drive named with -f\n";
    }
 
    if (! $to_tape && defined($opt_f)) {
	warn "warning (do-backups): ignoring -f option since we aren't backing up to tape\n";
	undef($opt_f);
    }

    if (defined($opt_f)) {
	($tape_user, $tape_host, $tape_dev) = &splittapedev($opt_f);
	$rsh = &make_rsh_cmd($tape_user, $tape_host);
	$drive_name = $tape_name{"$tape_host:$tape_dev"};
	$status_ok = $tape_status_ok{$drive_name};
	if ($tape_user ne "") {
	    $tape_user_host_dev = "$tape_user@$tape_host:$tape_dev";
	} else {
	    $tape_user_host_dev = "$tape_host:$tape_dev";
	}
    }

    if (defined($opt_t)) {
	$tape_id = $opt_t;
    } else {
	$tape_id = -1;
    }

    if (defined($opt_d)) {
	$debug_string = "-d $opt_d";
    } else {
	$debug_string = "";
    }

    $run_string = "-r $opt_r";
}

#
# See if its ok to do this chain, level of backup.  If no chains were
# mentioned in the backups file, then its ok.  If chains were mentioned, 
# and this chain was not, then its skipped.  If this chain was mentioned, 
# then check the level.  If no level was given, all levels should be done
# so we're cool.  If a level was mentioned and this level is greater than
# that level, skip it.  Otherwise its ok.  Got it?
#
sub ok_to_do_this_backup {
    local($chain, $level, *chain_list) = @_;
    local($found_this_chain, $found_any_chain, $tmp_chain, $tmp_level);

    $found_this_chain = 0;
    $ok_to_do = 1;		# if no chains given, assume ok

  chain_loop:
    while (($tmp_chain, $tmp_level) = each %chains) {
	$ok_to_do = 0;		# oops, gave a chain, can't asume its ok
	$found_any_chain = 1;

	if ($chain == $tmp_chain) {
	    $found_this_chain = 1;
	    if ($tmp_level == -1) {
		# No level given for this chain, assume ok
		$ok_to_do = 1;
	    } elsif ($level <= $tmp_level) {
		$ok_to_do = 1;
	    }
	    last chain_loop;
	}
    }

    return($ok_to_do);
}

sub update_buid_flag {
    local($name, $value) = @_;

    %replace = ();
    $replace{$name} = $value;

    if (&update_db($backup_host_file . $hostname, $backup_host_file . $hostname,
		   $buid,"backup_host", $DB_UPDATE, *replace, *add, *result)) {
        &sigh("error ($0): cannot create DB entry for $buid in backup host file for $hostname\n");
    }
}

#
# should we skip this backup?  I dunno, let's go check.  Returns true if this 
# backup should be skipped, false otherwise.
# 

sub should_skip_this_backup {
    #
    # Look for old-style /etc/backups files, and bitch about them.
    #
    if ($archive_type eq "" || $archive_type =~ /[0-9].*/) {
        warn "warning (do-backups): old style entry in $backup_file on line $. Skipping this entry!\n";
	return(1);
    }

    #
    # check the archive type - if none, don't do it...
    #
    if ($archive_type eq "none") {
	return(1);
    }

    #
    # Next entry if this one's marker matches the -m option (if that 
    # was given).
    #
    if (defined($opt_x) && &find_element_in_list($opt_x, @tags)) {
	warn "skipping since this directory's tag is excluded\n" 
	  if $opt_d & $DEBUG_INFO;
	return(1);
    }

    if (defined($opt_i) && ! &find_element_in_list($opt_i, @tags)) {
	warn "skipping since this directory's tag is not included\n" 
	  if $opt_d & $DEBUG_INFO;
	return(1);
    }

    if (! &ok_to_do_this_backup($chain, $level, *chains)) {
	warn "skipping since this chain and level are supposed to be skipped\n"
	  if $opt_d & $DEBUG_INFO;
	return(1);
    }

    return(0);
}

