#
# Copyright (c) 1990,1991,1992 The Ohio State University.
# All rights reserved.
#
# Redistribution and use in source and binary forms are permitted
# provided that: (1) source distributions retain this entire copyright
# notice and comment, and (2) distributions including binaries display
# the following acknowledgement:  ``This product includes software
# developed by The Ohio State University and its contributors''
# in the documentation or other materials provided with the distribution
# and in all advertising materials mentioning features or use of this
# software. Neither the name of the University nor the names of its
# contributors may be used to endorse or promote products derived
# from this software without specific prior written permission.
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#

#
# Various sorting routines.
#

#
# sort list of backups chronologically.
#
sub chrono {
    return(-($backup_times{$a} - $backup_times{$b}));
}

#
# Sort an array of integers into increasing order
#
sub increasing {
    ($a+0) - ($b+0);
}

#
# Sort tape IDs into increasing order (by prefix, and then within 
# each prefix by increasing number.)
#
sub increasing_tapeid {
    local($prefix_a, $prefix_b, $num_a, $num_b);

    if ($a =~ m/(.*)-([0-9]+)/) {
	$prefix_a = $1;
	$num_a = $2 + 0;
    } else {
	$prefix_a = "";
	$num_a = $a + 0;
    }

    if ($b =~ m/(.*)-([0-9]+)/) {
	$prefix_b = $1;
	$num_b = $2 + 0;
    } else {
	$prefix_b = "";
	$num_b = $b + 0;
    }

    return(1) if $prefix_a gt $prefix_b;
    return(-1) if $prefix_a lt $prefix_b;
    return($num_a - $num_b);
}

#
# Get the date
#
sub getdate {
    local($now);

    open(DATE, "$date_prog |") || &sigh("can't get date");
    chop($now = <DATE>);
    close(DATE);
    $now;
}

#
# Construct a tape ID given the config_name, the type, and a number.
#

sub make_tape_id_from_number {
    local($type, $num, $prefix) = @_;
    local($config);

    #
    # If the prefix is not "", use that and just append the number.  
    # Otherwise, construct an id from the config name (if set), 
    # the type and the number.
    #
    if ($prefix ne "") {
	return("$prefix-$num");
    } else {
	if ($config_name ne "") {
	    $config = "$config_name-";
	} else {
	    $config = "";
	}

	return("$config$type-$num");
    }
}

#
# Prints the fields from the given tape database entry and looks up
# additional info in the per host tape and  databases.  
#

sub print_run_entry {
    local($key, %run_entry) = @_;
    local(%RUNHOSTDB, %BUHOSTDB);
    local($runhost_value, %runhost_array, $buhost_value, %buhost_array);
    local($host, $buid, $start, $done, $scheduled);

    $ENV{"TZ"} = $our_timezone;

    chop($start = &ctime($run_entry{"start_date"}+0));
    chop($done = &ctime($run_entry{"done_date"}+0));

    printf "run %-6d chain %d level %d flags: %s active: %s\n hosts: %s\n",
	   $key,
	   $run_entry{"chain"},
	   $run_entry{"level"},
	   $run_entry{"flags"},
	   $run_entry{"active"},
	   $run_entry{"hosts"};
    if ($run_entry{"flags"} eq "scheduled") {
	printf " scheduled at: $start to start: %s\n",
	    $run_entry{"scheduled_for"};
	return;
    } else {
	printf " started: $start";
	if ($run_entry{"done_date"}+0 != 0) {
	    printf " done: $done\n";
	} else {
	    print "\n";
	}
    }

  host_loop:
    foreach $host (split(/,/, $run_entry{"hosts"})) {
	if (! &open_db(*RUNHOSTDB, $run_host_file . $host, $db_mode)) {
	    warn sprintf("warning: can't open the database %s on %s in %s",
			 $run_host_file . $host,
			 $hostname,
			 $0);
	    next host_loop;
	}

	if (! &open_db(*BUHOSTDB, $backup_host_file . $host, $db_mode)) {
	    warn sprintf("warning: can't open the database %s on %s in %s",
			 $backup_host_file . $host,
			 $hostname,
			 $0);
	    next host_loop;
	}

	if (! &read_db(*RUNHOSTDB, $key, *runhost_value, *runhost_array)) {
	    printf("  %s -- no run database entry\n", $host);
	    next host_loop;
	}

      buid_loop:
	foreach $buid (sort increasing split(/,/, $runhost_array{"buid_list"})) {
	    if (! &read_db(*BUHOSTDB, $buid, *buhost_value, *buhost_array)) {
		printf("  %s -- no backup database entry for backup ID %d\n", 
		       $host, 
		       $buid);
		next buid_loop;
	    }

	    printf("  %-10s buid: %-4d tape: %15s status: %-9s %s\n",
		   $host,
		   $buid,
		   $buhost_array{"tflist"},
		   $buhost_array{"flags"},
		   $buhost_array{"fs"});
	    if (defined($buhost_array{"afile"})) {
		printf("    archive: %s status: %s\n",
		       $buhost_array{"afile"},
		       $buhost_array{"d-flags"});
	    }
	    if (defined($opt_v)) {
		$error_name = sprintf("%s/%s.%d", 
				      $error_dir,
				      $host,
				      $buid);
		if (-r $error_name) {
		    open(FOO, $error_name) || 
		      warn "warning ($0): can't open $error_name ($!)\n";

		    while (<FOO>) {
			printf("    %s", $_);
		    }
		    close(FOO);
		}
	    }
	}

	&close_db(*BUHOSTDB, $backup_host_file);
	&close_db(*RUNHOSTDB, $run_host_file);
    }
}


#
# Prints the fields from the given tape database entry 
#

sub print_tape_entry {
    local($key, %tape_entry) = @_;
    local($add_date, $used_date);
    local($location);

    chop($add_date = &ctime($tape_entry{"add_date"}));
    chop($used_date = &ctime($tape_entry{"used_date"}));

    if ($tape_entry{"location"} ne "") {
	$location = sprintf(" location: %s", $tape_entry{"location"});
    } else {
	$location = "";
    }
    
# Should look something like this:
# test-100 flag: promised run: xxxxx used: xxx type: qic-150 location: bermuda
#     add: Fri Feb 14  9:49:31 1992 used: Fri Feb 14  9:49:31 1992
#     drive: brachiosaur:/dev/nrst0


    if ($tape_entry{"run_id"} eq "*") {
	$run = "    *";
    } else {
	$run = sprintf("%5d", $tape_entry{"run_id"});
    }

    printf "%9s flag: %-8s run:%s used:%3d type: %s-%s%s\n",
	   $key,
	   $tape_entry{"flags"},
	   $run,
	   $tape_entry{"uses"},
	   $tape_entry{"type"},
	   $tape_entry{"format"},
           $location;
    if (defined($opt_v)) {
	printf "    added: %s last use: %s\n    drive: %s\n",
	       $add_date,
	       $used_date,
               $tape_entry{"drive"};
    }
}


#
# &my_system - use system to execute something, deal with errors and debugging 
# stuff.
#
sub my_system {
    local($command) = @_;
    local($result, $is_ok);

    print "system: '$command'\n" if $opt_d & $DEBUG_SYSTEM;
    $result = system($command);

    if ($result != 0) {
	printf(stderr "error ($0): command '$command' failed with status %d\n",
	       $command,
	       $result/256);
	$is_ok = &askdef("Continue", "no",
"There was a fatal error in some command, and this program wants to know
whether to carry on with its work or quit.  Since fatal errors aren't
supposed to happen, it might be safest to just type 'no'.  If you want to
continue, type 'yes'.\n");
	if ($is_ok eq "no") {
	    &sigh("");
	}
    }
}


#
# found_in_list looks for $what in array @list and returns 1 if it is found,
# and 0 otherwise.
#

sub found_in_list {
    local($what, @list) = @_;
    local($target);

    foreach $target (@list) {
	return(1) if $what eq $target;
    }

    return(0);
}

#
# find_element_in_list splits $what on ,'s and looks for each element of $what
# in array @list and returns 1 if it is found, and 0 otherwise.
#
sub find_element_in_list {
    local($what, @list) = @_;
    local($target);
    local(@elements) = split(/,/,$what);

    foreach $target (@list) {
	foreach $element (@elements) {
		return(1) if $element eq $target;
	}
    }

    return(0);
}

sub short_date {
    local($tstamp) = @_;
    local(@local_time);

    @local_time = localtime($tstamp);
    return(sprintf("%d/%d/%2.2d %2d:%2.2d:%2.2d",
		   $local_time[4]+1,
		   $local_time[3],
		   $local_time[5] % 100,
		   $local_time[2],
		   $local_time[1],
		   $local_time[0]));
}

#
# turn an archive file reference into a host name and file name.
#
sub get_archive_host_and_file {
    local($afile) = @_;
    local($host, $file);

    if ($afile =~ m/:/) {
	($host, $file) = split(/:/, $afile);
    } else {
	$host = $hostname;
	$file = $afile;
    }

    return($host, $file);
}

1;
