#
# Copyright (c) 1990,1991,1992 The Ohio State University.
# All rights reserved.
#
# Redistribution and use in source and binary forms are permitted
# provided that: (1) source distributions retain this entire copyright
# notice and comment, and (2) distributions including binaries display
# the following acknowledgement:  ``This product includes software
# developed by The Ohio State University and its contributors''
# in the documentation or other materials provided with the distribution
# and in all advertising materials mentioning features or use of this
# software. Neither the name of the University nor the names of its
# contributors may be used to endorse or promote products derived
# from this software without specific prior written permission.
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#

#
# Check to see that the named tape is in the named drive.  Bitch and 
# moan to someone if it isn't.
#
# Steve Romig, February 14, 1992.
#

require 'global.defs';
require 'local.defs';
require $yagrip_perl;
require $backuplib_perl;

$options = "ht:d:f:";
$usage = "usage: check-tape [-t tape] [-d debuglevel] [-f host:dev] [-h]
    -t tape		gives ID of tape we expect to find in the drive
    -f host:device	specifies the tape drive to check
    -d debuglevel	turns on debugging output
    -h			display this message\n";

    #
    # Make sure that we clean up after ourselves if someone wants to kill us.
    #
$SIG{'INT'} = 'handle_interrupts';
$SIG{'HUP'} = 'handle_interrupts';
$SIG{'TERM'} = 'handle_interrupts';
$SIG{'QUIT'} = 'handle_interrupts';

&read_config();

    #
    # Deal with arguments...
    #
die $usage if ! &getopt($options);
die $usage if $#ARGV != -1;
die $usage if ! defined($opt_t) || ! defined($opt_f);
die $usage if defined($opt_h);

$tape_id = $opt_t;
($tape_user, $tape_host, $tape_dev) = &splittapedev($opt_f);
$rsh = &make_rsh_cmd($tape_user, $tape_host);

if (! &my_verify_tape($tape_id, $tape_host, $tape_dev, $rsh)) {
    $cmd = "| $bitch_prog";

    open(BITCH, $cmd) || die 
"error (tape-check): cannot execute '$cmd' to complain.  Tape $tape_id should be mounted in drive $tape_host:$tape_dev, but doesn't appear to be:
    $reason\n";

    print BITCH "$reason

Tape $tape_id should be mounted in drive $tape_host:$tape_dev, but doesn't
appear to be.  

Please check into it.  Thanks!

--- your friendly backup system.\n";

    close(BITCH);
}

exit 0;


#
# verify_tape checks to see that tape number $tape_id is mounted on
# drive $tapedev on host $tapehost.
#
# This is slightly different than the one in backuplib - there, we 
# assume that we are allowed to "write" to the tape drive, so we can
# rewind and read with impunity.  Here, we aren't quite so certain that 
# the tape is the writeable tape, and we don't want to modify its position
# more than necessary to determine that it is/isn't the correct tape.
#
sub my_verify_tape {
    local($tape_id, $tape_host, $tape_dev, $rsh) = @_;
    local($this_id, $file_number, $block_number);
    local($this_tape_type, $drive_name);

    open(SAVERR, ">&STDERR");
    open(STDERR, ">&STDOUT");

    $drive_name = $tape_name{"$tape_host:$tape_dev"};
    if ($tape_status_ok{$drive_name}) {
	#
	# If this tape host has an 'mt status' that works right, 
	# then get the current file and block number.  We do this to 
	# avoid reading the tape and screwing up the tape position in 
        # case this is the wrong tape. 
	#
	if (&tape_position($rsh, $tape_dev, *file_number, *block_number)) {
	    $reason = "couldn't get device status - no tape loaded or drive in use.";
	    return(0);
	}

	if ($file_number == -1) {
	    $reason = "no tape loaded?";
	    return(0);
	}

	if ($block_number != 0 || $file_number != 0) {
	    $reason = "tape isn't positioned at the beginning - in use?";
	    return(0);
	}
    }

    #
    # There's a tape, its at BOT, read the label...
    #
    if (&tape_get_id($rsh, $tape_dev, *this_id)) {
	$reason = "Can't open the tape to read the label.  Is it in use?";
	return(0);
    };

    #
    # Rewind the tape
    #
    &tape_rewind($rsh, $tape_dev);

    #
    # If the ID is just a number, then its an old fashioned ID.  Turn it
    # into a modern tape ID and then do the compare...
    #
    if ($this_id =~ m/^$config_name-([0-9]+)$/o ||
	$this_id =~ m/^([0-9]+)$/o) {
	$this_tape_type = $tape_type{$tape_name{"$tape_host:$tape_dev"}};

	$this_id = &make_tape_id_from_number($this_tape_type, $1 + 0, "");
    }
	
    if ($this_id eq $tape_id) {
        return 1;
    }

    $reason = "Drive $tape_host:$tape_dev contains tape $this_id, not tape $tape_id.";
    return(0);
}

